import numpy
import pandas
from scipy.stats import pearsonr


def learn(learning_data):
    x = learning_data['ERA5'].values
    y = learning_data['OBSGAP'].values
    p = numpy.polyfit(x, y, 1)
    k = float(p[0])
    n = float(p[1])
    pc, p_value = pearsonr(x, y)
    print("k = %f, n = %f, Pearson's correlation coefficient = %f, 2-tailed p-value = %g" % (k, n, pc, p_value))
    return k, n


def fill_gaps(data, diurnal_bias_filter=False):
    data['DEB'] = numpy.nan
    missing_dates = data[data['OBSGAP'].isnull()]
    for missing_date in missing_dates.index:
        learning_period_start = missing_date - pandas.to_timedelta('30day')
        learning_period_end = missing_date - pandas.to_timedelta('1hour')
        learning_data = data[learning_period_start:learning_period_end]
        if diurnal_bias_filter:  # subset same part of day
            time_window = 2
            start = (missing_date.hour - time_window + 24) % 24
            end = (missing_date.hour + time_window) % 24
            learning_data = learning_data.between_time('%d:%d' % (int(start), (start - int(start)) * 60),
                                                       '%d:%d' % (int(end), (end - int(end)) * 60))
        k, n = learn(learning_data)
        p = numpy.poly1d((k, n))
        value = p(data.ix[missing_date]['ERA5'])
        data.ix[missing_date]['OBSGAP'] = value
        data.ix[missing_date]['DEB'] = value
    return data
